// By Mario Brito - mariobrito2@gmail.com

using System;
using System.Text;

namespace p2huff.Huffman
{
	public class HuffDecode
	{
		Tree tree;
		
		public HuffDecode (Tree tree)
		{
			if (tree == null) throw new MalformedHuffmanTreeException();
			this.tree = tree;
		}
		
		public string decStream(BitStream stream)
		{
			StringBuilder str = new StringBuilder(200*1024);
			
			int bitLen = sizeof(UInt32) * 8;
			UInt32 bits = stream.popBits(ref bitLen);
			
			while (bitLen > 0)
			{
				int consumed = 0;
				char produced = tree.decode(bits, ref consumed);
				
				if (consumed > 0)
				{
					str.Append(produced);
					bits <<= consumed;
					bitLen -= consumed;
					
					int read = consumed;
					UInt32 moreBits = stream.popBits(ref read);
					bitLen += read;
					
					if (read < consumed)
					{
						// getting close to the end of the bit stream... pad it with zeros as needed
						moreBits <<= (consumed - read);
					}
					
					bits |= moreBits;
				}
				else
				{
					break;
				}
				
			}
			
			return str.ToString();
		}
	}
}

