// By Mario Brito - mariobrito2@gmail.com

using System;
namespace p2huff
{
	public class HuffmanChunk
	{
		byte[] chunk;
		int treeStart;
		int streamStart;
		
		int treeLength;
		int streamLength;
		
		public HuffmanChunk (byte[] chunk)
		{
			this.chunk = chunk;
			
			this.treeStart = 
				((chunk[0] & 0xFF) <<  0) |
				((chunk[1] & 0xFF) <<  8) |
				((chunk[2] & 0xFF) << 16) |
				((chunk[3] & 0xFF) << 24);

			this.streamStart = 
				((chunk[ 8] & 0xFF) <<  0) |
				((chunk[ 9] & 0xFF) <<  8) |
				((chunk[10] & 0xFF) << 16) |
				((chunk[11] & 0xFF) << 24);
		
			this.treeLength = streamStart - treeStart;
			this.treeLength = (this.treeLength >> 2) << 2;
			
			this.streamLength = chunk.Length - streamStart;
		}		
		
		public byte[] getTreeBytes()
		{
			if (treeLength > 0)
			{
				byte[] ret = new byte[treeLength - 2 * sizeof(UInt32)]; 
				System.Array.Copy(chunk, treeStart + 2 * sizeof(UInt32), ret, 0, treeLength - 2 * sizeof(UInt32));
				return ret;
			}
			
			throw new MalformedHuffmanTreeException();
		}
		
		public byte[] getStream()
		{
			if (streamLength > 0)
			{
				byte[] ret = new byte[streamLength]; 
				System.Array.Copy(chunk, streamStart, ret, 0, streamLength);
				return ret;
			}
			
			throw new MalformedHuffmanTreeException();
		}
	}
}