// By Mario Brito - mariobrito2@gmail.com

using System;

namespace p2huff.Huffman
{
	public class Tree
	{
		Node[] tree;
		
		public Tree (byte[] treeNodeBytes)
		{			
			if (treeNodeBytes != null && treeNodeBytes.Length > 0 && (treeNodeBytes.Length & 3) == 0)
			{
				int[] nodes = new int[treeNodeBytes.Length / 4];
				
				for (int i = 0, j = 0; i < treeNodeBytes.Length; i += 4, ++j)
				{
					nodes[j] =  ((treeNodeBytes[i + 0] & 0xFF) << 0)  |
								((treeNodeBytes[i + 1] & 0xFF) << 8)  |
								((treeNodeBytes[i + 2] & 0xFF) << 16) |
								((treeNodeBytes[i + 3] & 0xFF) << 24);
				}
				
				tree = new Node[nodes[0] + 1];
				tree[nodes[0]] = new Node(nodes[0]);	// root
				for (int i = nodes.Length - 1; i >= 1; --i)
				{
					insertNode(nodes[i]);
				}
			}
			else
			{
				throw new MalformedHuffmanTreeException();
			}
		}
		
		private void insertNode(int nodeNum)
		{
			for (int i = tree.Length - 1; i >= 0; --i)
			{
				if (tree[i] != null)
				{
					for (int j = 0; j < tree[i].children.Length; ++j)
					{
						if (tree[i].children[j] == Node.EMPTY)
						{
							tree[i].children[j] = nodeNum;
							
							if (tree[nodeNum] == null)
							{
								tree[nodeNum] = new Node(nodeNum);
							}
							
							return;
						}
					}
				}
			}
			throw new MalformedHuffmanTreeException();
		}
		
		private const int INBITS = 32;
		public char decode(UInt32 bits, ref int consumed)
		{
			char symbol = (char) 0;
			if (tree != null && tree.Length > 0)
			{
				int node = tree.Length - 1;				
				for(int i = 0; i < INBITS; ++i)
				{
					if (node < 256)
					{
						symbol = (char) node;
						consumed = i;
						return symbol;
					}
					else
					{
						UInt32 branch = ( bits >> (INBITS - 1) ) & 1;
						bits <<= 1;
						node = tree[node].children[branch];
					}
				}
			}
			
			throw new MalformedHuffmanTreeException();
		}
	}
}

