// By Mario Brito - mariobrito2@gmail.com

using System;
using System.IO;
using p2huff.Huffman;
using p2huff.IFF;

namespace p2huff
{
	class MainClass
	{
		public static void Main (string[] args)
		{
			bool pretty = false;
			string fileName = null;
			UInt32 offset = 0;
			
			Console.WriteLine("Privateer 2 Huffman Text Decompressor - By Mario Brito");
			Console.WriteLine();
			if (args.Length < 2)
			{
				Console.WriteLine("Usage is: p2huff <file name> <offset> [-pretty]");
				Console.WriteLine("Example: p2huff booth.iff 7917400 -pretty");
				Console.WriteLine();
				return;
			}			
			
			for (int i = 0, j = 0; i < args.Length; ++i)
			{
				if (args[i].Equals("-pretty"))
				{
					pretty = true;
				}
				else
				{
					switch(j)
					{
						case 0:
							fileName = args[i];
							++j;
							break;
						
						case 1:
							offset = UInt32.Parse(args[i]);
							++j;
							break;
						
						default:
							break;
					}
				}
			}
				
			FileStream fs = null;
			try
			{
				fs = File.OpenRead(fileName);			
				fs.Seek(offset, SeekOrigin.Begin);

				// expects stream to be at the start of the chunk (to be improved...)
				ChunkReader chunkReader = new ChunkReader(fs);
				byte[] txt2 = chunkReader.read("TXT2");
				
				// get tree and compressed stream from the chunk we just read.
				HuffmanChunk chunk = new HuffmanChunk(txt2);
				Tree tree = new Tree(chunk.getTreeBytes());
				BitStream bitStream = new BitStream(chunk.getStream());
				
				// decompress 
				// (later: maybe improve this part and actually follow the index instead of dumping everything?)
				HuffDecode decode = new HuffDecode(tree);
				string text = decode.decStream(bitStream);
				
				if (pretty)
				{
					for (int i = 128; i < 256; ++i)
					{
						text = text.Replace("" + ((char) i), "<" + i + ">\r\n");
					}
				}
				
				System.Console.WriteLine(text);
			}
			catch (Exception e)
			{
				System.Console.WriteLine("ERROR, aborting decompression! Exception was: " + e.ToString());
				throw;
			}
			finally
			{
				if (fs != null)
				{
					fs.Close();
				}
			}
		}
	}
}

